@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: CustomConfig.pm 3638 2006-06-17 20:28:07Z sysjkf $
+#   $Id: CustomConfig.pm 4378 2008-04-06 16:07:16Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 3638 $, 10;
+$VERSION = substr q$Revision: 4378 $, 10;
 
 #
 # These are the custom functions that you can write to produce a value
@@ -764,7 +764,8 @@ my(@CIDRlist);    # Ordered list of rules, all in CIDR form
 my($hostname);
 
 my $ConfFile = $ARGV[0];
-$ConfFile = $ARGV[1] if $ConfFile =~ /^-+v/i;
+$ConfFile = $ARGV[1] if $ConfFile =~ /^-+/i;
+$ConfFile = '/opt/MailScanner/etc/MailScanner.conf' unless $ConfFile && -f $ConfFile;
 
 my ($AccessDB, $Refusal, $my_mta); # Generalized here for Multiple MTA support
 $my_mta = lc(MailScanner::Config::QuickPeek($ConfFile, 'mta')) if $ConfFile &&
@@ -781,14 +782,13 @@ if ($my_mta eq "postfix") {
   $Refusal = "";
 }
 
-
 sub InitIPBlock {
   my($LimitsH, $cidr, $limit, $counter);
 
   # Skip IP block initialization if not postfix or sendmail
   if ($my_mta !~ /postfix|sendmail/) {
-    MailScanner::Log::ErrorLog("IPBlock Currently not supported " .
-                               "for your MTA %s", $my_mta);
+    MailScanner::Log::WarnLog("IPBlock Currently not supported " .
+                              "for your MTA %s", $my_mta);
     return 1;
   }
 
@@ -864,8 +864,8 @@ sub IPBlock {
   # Skip IPblock if not sendmail or postfix
 
   if ($my_mta !~ /postfix|sendmail/) {
-    MailScanner::Log::ErrorLog("IPBlock Currently not supported for " .
-                               "your MTA %s", $my_mta);
+    MailScanner::Log::WarnLog("IPBlock Currently not supported for " .
+                              "your MTA %s", $my_mta);
     return 1;
   }
 
