@@ -253,7 +253,8 @@ sub ReadQf {
   my %metadata;
   my($InHeader, $InSubject, $InDel, @headers, $msginfo, $from, @to, $subject);
   my($ip, $sender, %acl, %aclc, %aclm, $line, $acltype);
-  my($read1strcvd, $ipfromheader);
+  my(@rcvdiplist);
+  #my($read1strcvd, $ipfromheader);
 
   #print STDERR "ReadQf for " . $message->{id} . "\n";
 
@@ -582,17 +583,21 @@ sub ReadQf {
 	}
         if ("received:" eq lc $1) {
           my $received = $2;
+          my $rcvdip = '127.0.0.1';
           if ($received =~ /\[(\d+\.\d+\.\d+\.\d+)\]/i) {
-            unless ($read1strcvd) {
-              $ipfromheader = $1;
-              $read1strcvd = 1;
-            }
+            $rcvdip = $1;
+            #unless ($read1strcvd) {
+            #  $ipfromheader = $1;
+            #  $read1strcvd = 1;
+            #}
           } elsif ($received =~ /\[([\dabcdef.:]+)\]/i) {
-            unless ($read1strcvd) {
-              $ipfromheader = $1;
-              $read1strcvd = 1;
-            }
+            $rcvdip = $1;
+            #unless ($read1strcvd) {
+            #  $ipfromheader = $1;
+            #  $read1strcvd = 1;
+            #}
           }
+          push @rcvdiplist, $rcvdip;
         }
       }
       # Track anything we may be in the middle of
@@ -609,8 +614,13 @@ sub ReadQf {
   }
 
   # If we were told to read the IP from the header and it was there...
-  $message->{clientip} = $ipfromheader
-    if $getipfromheader && $read1strcvd && $ipfromheader ne "";
+  $getipfromheader = @rcvdiplist if $getipfromheader>@rcvdiplist;
+  # If they wanted the 2nd Received from address, give'em element 1 of list
+  $message->{clientip} = $rcvdiplist[$getipfromheader-1] if
+    $getipfromheader>0;
+
+  #$message->{clientip} = $ipfromheader
+  #  if $getipfromheader && $read1strcvd && $ipfromheader ne "";
 
   # We should have the last header built but not pushed
   # onto the metadata headers array at this point...
@@ -816,7 +826,12 @@ sub AddHeader {
   $newvalue =~ s/\n*\Z/\n/;
 
   $newheader = { name => $newkey, body => $newvalue, flag => " " };
-  push @{$message->{metadata}{headers}}, $newheader;
+  # DKIM: Add header at top if adding headers at top
+  if ($message->{newheadersattop}) {
+    unshift @{$message->{metadata}{headers}}, $newheader;
+  } else {
+    push @{$message->{metadata}{headers}}, $newheader;
+  }
 
   return 1;
 }
@@ -830,13 +845,24 @@ sub AddHeader {
 sub DeleteHeader {
   my($this, $message, $key) = @_;
 
+  my $usingregexp = ($key =~ s/^\/(.*)\/$/$1/)?1:0;
+
+  # Add a colon if they forgot it.
+  $key .= ':' unless $usingregexp || $key =~ /:$/;
+  # If it's not a regexp, then anchor it and sanitise it.
+  $key = quotemeta($key) unless $usingregexp;
+
   # Delete header by flagging it as deleted rather than by
   # actually deleting it; might help with debugging.
   # Also keep track of any flags that we've managed to "vanish".
-  my($hdrnum);
+  my($hdrnum, $line);
   my $metadata = $message->{metadata};
   for ($hdrnum=0; $hdrnum<@{$metadata->{headers}}; $hdrnum++) {
-    next unless lc $metadata->{headers}[$hdrnum]{name} eq lc $key;
+    # Skip if they are using a header name and it doesn't match
+    next if !$usingregexp && lc $metadata->{headers}[$hdrnum]{name} ne lc $key;
+    # Skip if they are using a regexp and it doesn't match
+    $line = $metadata->{headers}[$hdrnum]{name} . $metadata->{headers}[$hdrnum]{body};
+    next if $usingregexp && $line !~ /$key/i;
     # Have found the right line
     $metadata->{headers}[$hdrnum]{flag} ne " "
       and $metadata->{vanishedflags}{$metadata->{headers}[$hdrnum]{flag}} = 1;
@@ -874,7 +900,8 @@ sub UniqHeader {
 sub ReplaceHeader {
   my($this, $message, $key, $newvalue) = @_;
 
-  $this->DeleteHeader($message, $key);
+  # DKIM: Don't do DeleteHeader if adding all headers at top
+  $this->DeleteHeader($message, $key) unless $message->{dkimfriendly};
   $this->AddHeader($message, $key, $newvalue);
 
   return 1;
@@ -1523,6 +1550,7 @@ sub CreateBatch {
     }
 
     $batchempty = 1;
+    my $maxattempts = MailScanner::Config::Value('procdbattempts');
 
     # Keep going until end of dir or have reached every imposed limit. This
     # now processes the files oldest first to make for fairer queue cleanups.
@@ -1554,12 +1582,74 @@ sub CreateBatch {
       # Lock and read the qf file. Skip this message if the lock fails.
       $newmessage = MailScanner::Message->new($id, $queuedirname,
                                               $getipfromheader);
-      if ($newmessage eq 'INVALID') {
+      if ($newmessage && $newmessage->{INVALID}) {
+      #if ($newmessage eq 'INVALID') {
         $invalidfiles .= "$id ";
+        undef $newmessage;
         next;
       }
       next unless $newmessage;
 
+      # JKF 20090301 Skip this message if $id has been scanned
+      # too many times.
+      # JKF 20090301 Read the number of times this message id has
+      # been processed. If over the limit, then ignore it.
+      # JKF 20090301 Should ideally make it in a batch of its own,
+      # and try that once or twice, *then* skip it. But let's try
+      # a simple version first.
+      # Just do a "next" if we want to skip the message.
+      if ($maxattempts) {
+        my $nexttime = time + 120 + int(rand(240)); # 4 +- 2 minutes
+        #my $nexttime = time + 10 + int(rand(20)); # 4 +- 2 minutes
+        my @attempts = $MailScanner::ProcDBH->selectrow_array(
+                         $MailScanner::SthSelectRows, undef, $id);
+        #my @attempts = $MailScanner::ProcDBH->selectrow_array(
+        #"SELECT id,count,nexttime FROM processing WHERE (id='" . $id . "')");
+        #print STDERR "id       = \"$attempts[0]\"\n";
+        #print STDERR "count    = \"$attempts[1]\"\n";
+        #print STDERR "nexttime = \"$attempts[2]\"\n";
+        if (@attempts && $attempts[1]>=$maxattempts) {
+          MailScanner::Log::WarnLog("Warning: skipping message %s as it has been attempted too many times", $id);
+          # JKF 20090301 next;
+          # Instead of just skipping it, quarantine it and notify
+          # the local postmaster.
+          $newmessage->QuarantineDOS();
+          #print STDERR "Moving $attempts[0], $attempts[1], $attempts[2] into archive table\n";
+          $MailScanner::SthDeleteId->execute($id);
+          #$MailScanner::ProcDBH->do(
+          #  "DELETE FROM processing WHERE id='" . $id . "'");
+          $MailScanner::SthInsertArchive->execute($attempts[0], $attempts[1],
+                                             $attempts[2]);
+          #$MailScanner::ProcDBH->do(
+          #  "INSERT INTO archive (id, count, nexttime) " .
+          #  "VALUES ('$attempts[0]', $attempts[1], $attempts[2])");
+        } elsif (defined $attempts[1]) {
+          # We have tried this message before
+          if (time>=$attempts[2]) {
+            # Time for next attempt has arrived
+            $MailScanner::SthIncrementId->execute($nexttime, $id);
+            #$MailScanner::ProcDBH->do(
+            #  "UPDATE processing SET count=count+1, nexttime=$nexttime " .
+            #  " WHERE id='" . $id . "'");
+            MailScanner::Log::InfoLog(
+              "Making attempt %d at processing message %s",
+              $attempts[1]+1, $id);
+            #print STDERR "Incremented $id\n";
+          } else {
+            # Not time for next attempt yet, so ignore the message
+            $newmessage->DropFromBatch();
+            next;
+          }
+        } else {
+          # We have never seen this message before
+          $MailScanner::SthInsertProc->execute($id, 1, $nexttime);
+          #$MailScanner::ProcDBH->do(
+          #  "INSERT INTO processing (id, count, nexttime) " .
+          #  "VALUES ('" . $id . "', 1, $nexttime)");
+          #print STDERR "Inserted $id\n";
+        }
+      }
+
       $batch->{messages}{"$id"} = $newmessage;
       $newmessage->{mtime} = $mtime;
       $batchempty = 0;
