@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm 4442 2008-05-08 00:21:26Z sysjkf $
+#   $Id: MessageBatch.pm 4842 2009-05-28 10:39:00Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -42,11 +42,13 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 use DirHandle;
 use Time::HiRes qw ( time );
 use POSIX;
+use File::Temp qw ( tempfile tempdir );
+use DBI qw(:sql_types);
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4442 $, 10;
+$VERSION = substr q$Revision: 4842 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -204,7 +206,7 @@ sub print {
 sub RemoveDeletedMessages {
   my $this = shift;
 
-  my($id, $message, @badentries);
+  my($id, $message, @badentries, @deletedentries);
 
   my $deleteifnotdelivering = 0;
   $deleteifnotdelivering = 1
@@ -223,13 +225,19 @@ sub RemoveDeletedMessages {
     #MailScanner::Log::InfoLog("RemoveDeletedMessages: Deleting message $id")
     #  if $message->{deleted};
     #print STDERR "Message->deleted = " . $message->{deleted} . "and dontdeliver = " . $message->{dontdeliver} . "\n";
-    $message->DeleteMessage() if $message->{deleted} ||
-      ($message->{dontdeliver} && $deleteifnotdelivering);
+    if ($message->{deleted} ||
+        ($message->{dontdeliver} && $deleteifnotdelivering)) {
+      $message->DeleteMessage();
+      push @deletedentries, $id;
+    }
+
     #delete $this->{messages}{$id} if $this->{messages}{$id}->{deleted};
   }
   foreach $id (@badentries) {
     delete $this->{messages}{$id};
   }
+  # Add to the list of wiped messages
+  $this->{deleted} .= ' ' . join(' ', @badentries, @deletedentries);
 }
 
 
@@ -508,17 +516,17 @@ sub Explode {
 sub VirusScan {
   my $this = shift;
 
+  #MailScanner::Log::InfoLog("Other Checks: Starting");
+  my $others  = MailScanner::SweepOther::ScanBatch($this, 'scan');
+  MailScanner::Log::NoticeLog("Other Checks: Found %d problems", $others+0)
+    if defined $others && $others>0;
+
   # Call them with the scanning settings
   MailScanner::Log::InfoLog("Virus and Content Scanning: Starting");
   my $viruses = MailScanner::SweepViruses::ScanBatch($this, 'scan');
   MailScanner::Log::NoticeLog("Virus Scanning: Found %d viruses", $viruses+0)
     if defined $viruses && $viruses>0;
 
-  #MailScanner::Log::InfoLog("Other Checks: Starting");
-  my $others  = MailScanner::SweepOther::ScanBatch($this, 'scan');
-  MailScanner::Log::NoticeLog("Other Checks: Found %d problems", $others+0)
-    if defined $others && $others>0;
-
   #MailScanner::Log::InfoLog("Content Checks: Starting");
   my $content  = MailScanner::SweepContent::ScanBatch($this, 'scan');
   MailScanner::Log::NoticeLog("Content Checks: Found %d problems", $content+0)
@@ -634,7 +642,7 @@ sub PrintInfectedSections {
   my($id, $message);
   my($parts, $file, $entity);
 
-  print STDERR "\nInfected sections are:\n";
+  #print STDERR "\nInfected sections are:\n";
   while(($id, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
     $parts = $message->{virusreports};
@@ -1166,7 +1174,8 @@ sub CreateEicarBatch {
   # Message number = 1
   # Path = irrelevant as we're not actually reading anything
   # It's a fake that we simulate ==> 1
-  my $newmessage = MailScanner::Message->new(1, '/tmp', 1);
+  my $MessageDir = tempdir( 'MSlintXXXXXX', CLEANUP => 1);
+  my $newmessage = MailScanner::Message->new(1, $MessageDir, 0, 1);
   @{$newmessage->{headers}} = ();
   @{$newmessage->{to}} = ();
   @{$newmessage->{touser}} = ();
@@ -1197,13 +1206,15 @@ sub CreateEicarBatch {
   $newmessage->WriteHeaderFile();
 
   # Create a file of the body
-  my $fh = new FileHandle(">/tmp/MSLint.body.$$");
+  my($fh, $temporaryname);
+  ($fh, $temporaryname) = tempfile()
+    or die "Could not create temp file $temporaryname for test message, $!";
   print $fh <<EOFLint;
 WDVPIVAlQEFQWzRcUFpYNTQoUF4pN0NDKTd9JEVJQ0FSLVNUQU5EQVJELUFOVElWSVJVUy1URVNU
 LUZJTEUhJEgrSCo=
 EOFLint
   $fh->close();
-  $newmessage->{store}->{dpath} = "/tmp/MSLint.body.$$";
+  $newmessage->{store}->{dpath} = $temporaryname;
 
   # Add it to the batch
   $batch->{messages}{"1"} = $newmessage;
@@ -1212,5 +1223,103 @@ EOFLint
   umask 0077;
 }
 
+# JKF 20090301 Delete all entries from the processed database table
+# which have been successfully processed. Anything with the "abandoned"
+# property will not be removed, all other members of the batch will be.
+sub ClearOutProcessedDatabase {
+  my $this = shift;
+
+  # Don't forget the master switch!
+  return unless MailScanner::Config::Value('procdbattempts');
+
+  # Clear out the entries for all the messages in the batch that don't
+  # have "abandoned" set, and all the entries in "{deleted}" property
+  # of the batch as well, space-separated list.
+  my $sth = $MailScanner::SthDeleteId;
+  #ProcDBH->prepare("DELETE FROM processing WHERE (id=?)");
+  MailScanner::Log::DieLog("Database complained about this: %s. I suggest you delete your %s file and let me re-create it for you", $DBI::errstr, MailScanner::Config::Value("procdbname")) unless $sth;
+
+  my($id, $message, $count, %gotridof);
+  $count = 0;
+  while(($id, $message) = each %{$this->{messages}}) {
+    next unless $message;
+    # Skip this message if it succeeded or failed due to global failure,
+    # e.g. disk space.
+    next if $message->{abandoned};
+    # Delete this $message->{id} from the database table
+    #my $rows = $sth->execute($id);
+    #print STDERR "Delete1: Rows = $rows\n";
+    #$count++ if $rows;
+    $gotridof{$id} = 1;
+  }
+          
+  foreach $id (split " ", $this->{deleted}) {
+    next unless $id;
+    # Delete this $message->{id} from the database table
+    #my $rows = $sth->execute($id);
+    #print STDERR "Delete2: Rows = $rows\n";
+    #$count++ if $rows;
+    $gotridof{$id} = 1;
+  }
+
+  foreach $id (keys %gotridof) {
+    next unless $id;
+    #$sth->bind_param(1, "$id", SQL_VARCHAR);
+    $sth->execute("$id");
+    #$MailScanner::ProcDBH->do("DELETE FROM processing WHERE (id='$id')");
+  }
+  #$sth->finish;
+
+  $count = keys %gotridof;
+  MailScanner::Log::InfoLog("Deleted %d messages from processing-database",
+                             $count);
+}
+
+# If we are killed while processing a batch, then decrement the counter by 1
+# for each message in the batch. If that gives a counter of 0 then delete
+# the record from the database altogether.
+sub DecrementProcDB {
+  my($this) = @_;
+
+  return unless $this;
+  return unless $MailScanner::ProcDBH;
+
+  my($id, $message, @attempts);
+
+  my $selectsth = $MailScanner::SthSelectCount; #ProcDBH->prepare(
+  #                "SELECT count FROM processing WHERE (id=?)");
+  my $deletesth = $MailScanner::SthDeleteId; #ProcDBH->prepare(
+  #                "DELETE FROM processing WHERE (id=?)");
+  my $updatesth = $MailScanner::SthDecrementId; #ProcDBH->prepare(
+  #                "UPDATE processing SET count=count-1 WHERE (id=?)");
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    $selectsth->execute($id);
+    @attempts = $selectsth->fetchrow_array();
+    if (@attempts) {
+      # There is a record, so delete it or decrement it
+      if ($attempts[0]>1) {
+        # Decrement it
+        $updatesth->execute($id);
+      } else {
+        # Delete it
+        $deletesth->execute($id);
+      }
+    }
+  }
+
+  # Remove all the messages which have already been deleted from the batch.
+  foreach $id (split " ", $this->{deleted}) {
+    next unless $id;
+    # Delete this $message->{id} from the database table
+    $deletesth->execute($id);
+  }
+
+  # Clear up
+  #$selectsth->finish;
+  #$deletesth->finish;
+  #$updatesth->finish;
+}
+
 1;
 
