@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: PFDiskStore.pm 4206 2007-10-30 13:59:18Z sysjkf $
+#   $Id: PFDiskStore.pm 4759 2009-04-21 17:57:34Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use MailScanner::Config;
 use vars qw($VERSION @DeletesPending);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4206 $, 10;
+$VERSION = substr q$Revision: 4759 $, 10;
 
 # List of pending delete operations so we can clear up properly when killed
 @DeletesPending = ();
@@ -59,7 +59,7 @@ $VERSION = substr q$Revision: 4206 $, 10;
 #use vars qw($VERSION);
 #
 #### The package version, both in 1.23 style *and* usable by MakeMaker:
-#$VERSION = substr q$Revision: 4206 $, 10;
+#$VERSION = substr q$Revision: 4759 $, 10;
 
 # Attributes are
 #
@@ -326,6 +326,7 @@ sub WriteHeader {
     # Quietly drop the data structures of this message.
     $message->{deleted} = 1;
     $message->{gonefromdisk} = 1; # Don't try to delete the original
+    #20090421 $message->{abandoned} = 1; # Retry this message as it was ditched
     unlink $tfile; # Delete the new file from the queue
     MailScanner::Lock::unlockclose($Tf);
     return;
@@ -506,11 +507,19 @@ sub ReadBody {
   # Handle trackback -- This is the tricky one
   if ($configwords[1] =~ /tr[ua]/i) {
     #print STDERR "Trackback:\n";
-    while (${@{$body}}[scalar(@{$body})-1] !~ /^\s*$/) {
-      print "Line is ****" . ${@{$body}}[scalar(@{$body})-1] . "****\n";
+    my $i;
+    for ($i=(@{$body}-1); $i>=0; $i--) {
+      last if $body->[$i] =~ /^\s*$/i;
       pop @{$body};
-      #print STDERR ".";
     }
+    #my $bodysize = $#{@$body}-1;
+    #while (${@{$body}}[$bodysize+1] !~ /^\s*@/) { 
+    #while(${@{$body}}[$#{@$body}] !~ /^\s*$/) {
+      #print "Line is ****" . ${@{$body}}[scalar(@{$body})-1] . "****\n";
+    #  pop @{$body};
+      #print STDERR ".";
+    #}
+
     #print STDERR "\n";
     $b->Done();
     return;
@@ -829,6 +838,16 @@ sub Start {
     }
 
     while(($type, $data) = MailScanner::Sendmail::ReadRecord($$this{_handle})) {
+      # p record handling by Glenn fix on 2008-03-24
+      # assumption: ReadQf has already validated most of this queue file, so
+      # skip the sanity checks. Just do the seeks as needed and read the next
+      # record. Also, skip any deleted data records (type w). Could've done
+      # this with recursion:-).
+      while ($type eq 'p' || $type eq 'w') {
+        seek $$this{_handle}, $data+0, 0 if ($type eq 'p' && $data+0 > 0);
+        ($type,$data) = MailScanner::Sendmail::ReadRecord($$this{_handle});
+      }
+
       last if $type eq 'N' && $data eq "";
       last if $type eq 'X';
     }
