@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SA.pm 4645 2009-01-08 10:34:46Z sysjkf $
+#   $Id: SA.pm 4760 2009-04-21 21:03:34Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use Compress::Zlib;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4645 $, 10;
+$VERSION = substr q$Revision: 4760 $, 10;
 
 # Attributes are
 #
@@ -217,6 +217,8 @@ sub initialise {
             MailScanner::Log::InfoLog("Connected to SpamAssassin cache database");
             # Rebuild all the tables and indexes. The PrintError=>0 will make it
             # fail quietly if they already exist.
+            # Speed up writes at the cost of database integrity. Only tmp data!
+            $MailScanner::SA::cachedbh->do("PRAGMA default_synchronous = OFF");
             $MailScanner::SA::cachedbh->do("CREATE TABLE cache (md5 TEXT, count INTEGER, last TIMESTAMP, first TIMESTAMP, sasaysspam INT, sahighscoring INT, sascore FLOAT, saheader BLOB, salongreport BLOB, virusinfected INT)");
             $MailScanner::SA::cachedbh->do("CREATE UNIQUE INDEX md5_uniq ON cache(md5)");
             $MailScanner::SA::cachedbh->do("CREATE INDEX last_seen_idx ON cache(last)");
